//
//  Phobos.m
//  GengmeiDoctor
//
//  Created by Thierry on 16/1/26.
//  Copyright © 2016年 wanmeizhensuo. All rights reserved.
//

#import "Phobos.h"
#import <AdSupport/AdSupport.h>
#import "PhobosUtil.h"
#import "PhobosConfig.h"
#import "UIResponder+PhobosPV.h"
#import "PhobosUtil.h"
#import "PhobosCustomVisibleController.h"
#import <GMKit/UIDevice+Resolutions.h>
#import <GMCache/GMCache.h>
#import <objc/runtime.h>
static Phobos *_sharedClient;
static NSString *sdkVersion = @"1.3.1";

@interface Phobos ()
@property (strong, nonatomic) UIViewController *visibleController;
@property (strong, nonatomic) NSDateFormatter *dateFormatter;
@property (strong, nonatomic) NSString *appName;
@property (strong, nonatomic) NSString *channelId;
@property (strong, nonatomic) NSString *appVersion;
@property (strong, nonatomic) NSString *sessionId;
/* 每一条埋点数据的物理ID，自增，生命周期和sessionId相同。特别注意：在sessionOver的时候，要把他置为0 */
@property (assign, nonatomic) NSInteger serialId;

//@property (nonatomic, weak) dispatch_semaphore_t immediatelySemaphore;// 保障需要立即发送的埋点数据安全
//@property (nonatomic, weak) dispatch_semaphore_t normalSemaphore;// 保障普通埋点数据安全
@property (nonatomic, assign) NSInteger normalCount;// 记录普通埋点数量

@end

@implementation Phobos

static dispatch_semaphore_t _immediatelySemaphore;
static dispatch_semaphore_t _normalSemaphore;
static dispatch_queue_t _immediatelyQueue;
static dispatch_queue_t _normalQueue;

+ (Phobos *)clientWithAppName:(NSString *)appName channelId:(NSString *)channelId{
    Phobos.sharedClient.appName = appName;
    Phobos.sharedClient.channelId = channelId;
    return Phobos.sharedClient;
}

+ (void)initialize {
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _immediatelySemaphore = dispatch_semaphore_create(1);
        _normalSemaphore = dispatch_semaphore_create(1);
        _immediatelyQueue = dispatch_queue_create("immdiately", DISPATCH_QUEUE_CONCURRENT);
        _normalQueue = dispatch_queue_create("normal", DISPATCH_QUEUE_CONCURRENT);
    });
}

- (instancetype)init {
    if (self = [super init]) {
        self.appName = @"";
        self.channelId = @"";
        self.logEnabled = NO;
        self.userId = @"";
        self.netStatus = @"";
        self.currentCityId = @"";
        self.serverAPI = @"";
        self.greyType = @"";
        self.userType = [[NSMutableDictionary alloc] initWithCapacity:0];
        self.appVersion = [PhobosUtil getAppVersion];
        self.signingType = PhobosSigningTypeUndefined;
//        self.immediatelySemaphore = dispatch_semaphore_create(1);
//        self.normalSemaphore = dispatch_semaphore_create(1);
        
        NSMutableDictionary *dataDict = [[GMCache fetchObjectAtDocumentPathWithkey:PhobosNormalCacheKey] mutableCopy];
        for (NSArray *data in dataDict.allValues) {
            self.normalCount += data.count;
        }
        
        [self setupNotification];
        [self handleSessionStart];
        [self synchronizePhobosKey];
    }
    return self;
}

+ (id)allocWithZone:(struct _NSZone *)zone {
    return Phobos.sharedClient;
}

- (id)copyWithZone:(struct _NSZone *)zone {
    return Phobos.sharedClient;
}

+ (Phobos *)sharedClient{// 使用单例设计模式，保证Phobos只有一个实例，并提供了全局访问点
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _sharedClient = [[super allocWithZone:NULL] init];
    });
    return _sharedClient;
}

- (void)setAppName:(NSString *)appName channelId:(NSString *)channelId {
    self.appName = appName;
    self.channelId = channelId;
}

/**
 * disk下的PhobosHaveOpenApp只要存在，就把他取出来放到document下(注：当不支持7.6.16版本的时候，干掉这个方法)
 */
- (void)synchronizePhobosKey {
    if ([GMCache fetchObjectAtDiskWithkey:PhobosHaveOpenApp]) {
        NSString *prefePath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
        prefePath = [NSString stringWithFormat:@"%@/com.tumblr.TMDiskCache.WMCacheShared/%@",prefePath ,PhobosHaveOpenApp];
        NSURL *fileUrl = [NSURL fileURLWithPathComponents:@[prefePath]];
        BOOL written = [NSKeyedArchiver archiveRootObject:PhobosHaveOpenApp toFile:[fileUrl path]];
    }
}

- (void)dealloc{
    if (self) {
        [[NSNotificationCenter defaultCenter] removeObserver:self];
    }
}

- (void)setUserType:(NSMutableDictionary *)userType {
    if (userType == nil && userType.count == 0) {
        return;
    }
    NSArray *newKeys = userType.allKeys;
    NSArray *oldKeys = _userType.allKeys;
    for (NSString *newKey in newKeys) {
        if ([oldKeys containsObject:newKey]) {
            NSString *newValue = [[userType objectForKey:newKey] stringValue];
            NSString *oldValue = [[_userType objectForKey:newKey] stringValue];
            if (![newValue isEqualToString:oldValue]) {
                [_userType setObject:[userType objectForKey:newKey] forKey:newKey];
            }
        } else {
            [_userType setObject:[userType objectForKey:newKey] forKey:newKey];
        }
    }
}

/**
   在APP启动、从后台到前台的时候需要记录device_opened埋点
 */
- (void)handleEventDeviceOpened{

    /** 每次打开APP埋点 **/
    NSDictionary *dict = @{@"build_cpu_abi": [PhobosUtil currentDeviceCPUType],
                           @"cpu_count": [PhobosUtil currentDeviceCPUCount],
                           @"mac_address": [PhobosUtil getMacAddress],
                           @"phone_operator": [PhobosUtil getTelephonyInfo],
                           @"total_memory": [PhobosUtil getTotalMemorySize],
                           @"run_time": [PhobosUtil deviceRunTime],
                           @"uuid": [PhobosUtil deviceId],
                           @"build_version_release": [[UIDevice currentDevice] systemVersion],
                           @"build_model": [UIDevice deviceVersion],
                           };
    [Phobos track:@"device_opened" attributes:dict sendNow:YES];
    
    /** 第一次打开APP埋点 **/
    // 当不再支持7.6.15版本时，只保留
    if (![GMCache fetchObjectAtDocumentPathWithkey:PhobosHaveOpenApp]) {
        [Phobos track:@"device_activated" attributes:@{} sendNow:YES];
        [Phobos track:@"device_activated" attributes:@{} sendNow:NO];
        [GMCache storeObjectAtDocumentPathWithkey:PhobosHaveOpenApp object:PhobosHaveOpenApp];
    }
}

- (UIViewController *)visibleController {
    id target = self.getTopController();

    if ([target conformsToProtocol:NSProtocolFromString(@"PhobosCustomVisibleController")]) {
        target = [target performSelector:@selector(phobosVisibleController)];
    }
    return target;
}

#pragma mark - notification handler
/**
 *  @brief 设置对APP的通知监听
 *
 *  @since 0.0.1
 */
- (void)setupNotification{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleAppFinishLaunch:) name:UIApplicationDidFinishLaunchingNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleAppInForeground:) name:UIApplicationWillEnterForegroundNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleAppInBackgound:) name:UIApplicationDidEnterBackgroundNotification object:nil];
}

/**
 *  @brief 每次打开APP或返回前台，即Session开始的时候的处理
 *  @notification didFinishLaunch和willEnterForeground的时候都需要记录
 *
 *  @since 0.0.1
 */
- (void)handleSessionStart{
    _serialId = 0;
    _sessionId = [[NSUUID UUID] UUIDString];

    [GMCache storeObjectAtDocumentPathWithkey:PhobosBeginTime object:[PhobosUtil currentTime]];
}

/**
 *  @brief 应用打开时的处理
 *
 *  @since 0.0.1
 */
- (void)handleAppFinishLaunch:(id)sender{
    phobosLog(@"handleAppFinishLaunch");
    [self handleSessionStart];
    [self handleEventDeviceOpened];
}

/**
 *  @brief 应用进入前台的处理
 *
 *  @since 0.0.1
 */
- (void)handleAppInForeground:(id)sender{
    phobosLog(@"handleAppInForeground");
    [self handleSessionStart];
    [self handleEventDeviceOpened];
    [self sendImmediatelyNormalData];
    [self handlePVEventAppInForeground];
}

/**
 *  @brief 应用进入后台的处理
 *
 *  @since 0.0.1
 */
- (void)handleAppInBackgound:(id)sender{
    phobosLog(@"handleAppInBackgound");
    [self handlePVEventAppInBackgound];
    [self handleSessionOver];
    [self sendImmediatelyNormalData];
}

/**
 *  @brief 会话结束时候的处理
 *
 *  @since 0.0.1
 */
- (void)handleSessionOver{

    //进入后台的同时，把记录时间同步到服务端，把已使用时间清空
    double beginTime = [[GMCache fetchObjectAtDocumentPathWithkey:PhobosBeginTime] doubleValue];
    if (beginTime == 0) {
        return;
    }
    NSDate *date = [NSDate date];
    double endTime = [date timeIntervalSince1970];
    NSString *usedTime = [NSString stringWithFormat:@"%ld",(long)(endTime - beginTime)];
    NSDictionary *dict = @{@"duration":usedTime,
                           @"build_cpu_abi": [PhobosUtil currentDeviceCPUType],
                           @"cpu_count": [PhobosUtil currentDeviceCPUCount],
                           @"mac_address": [PhobosUtil getMacAddress],
                           @"phone_operator": [PhobosUtil getTelephonyInfo],
                           @"total_memory": [PhobosUtil getTotalMemorySize],
                           @"run_time": [PhobosUtil deviceRunTime],
                           @"uuid": [PhobosUtil deviceId],
                           @"build_version_release": [[UIDevice currentDevice] systemVersion],
                           @"build_model": [UIDevice deviceVersion],
                           };
    [Phobos track:@"on_app_session_over" attributes:dict];
    [GMCache removeObjectAtDocumentPathWithkey:PhobosBeginTime];
    //当前session结束之后，把id置为0
    _serialId = 0;
}

/**
 APP从后台到前台的时候，重新初始化pagename等信息
 @author zhaiguojun 16-10-11 in (null)
 */
- (void)handlePVEventAppInForeground {
    if (self.visibleController != nil) {
        [self onPVStart:self.visibleController];
    }
}

/**
 APP进到后台的时候，把当前pageview时间结束
 @author zhaiguojun 16-10-11 in (null)
 */
- (void)handlePVEventAppInBackgound {
    if (self.visibleController != nil) {
        [self onPVEnd:self.visibleController];
    }
}

#pragma mark - PV
- (void)onPVStart:(UIResponder<PhobosPVProtocol> *)page {
    // 必须在此处调用一下referer，因为onControllerStart
    [page initReferer];
    [page initReferrerIdIfNil];
    [page initReferrerTabName];
    page.inTime = [PhobosUtil currentTime];
}

- (void)onPVEnd:(UIResponder<PhobosPVProtocol> *)page {
    if (![PhobosUtil isNonEmpty:page.pageName] || !page.needLogPV) {
        return;
    }
    
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    @try {
        [dict setObject:[PhobosUtil currentTime] forKey:@"out"];
        [dict setObject:page.inTime?:@"" forKey:@"in"];
        [dict setObject:page.pageName forKey:@"page_name"];
        [dict setObject:page.businessId?:@"" forKey:@"business_id"];
        [dict setObject:page.referer?:@"" forKey:@"referrer"];
        [dict setObject:@(0) forKey:@"fake"];
        [dict setObject:page.referrerId ? : @"" forKey:@"referrer_id"];
        [dict setObject:page.extraParam ? : @"" forKey:@"extra_param"];
        [dict setObject:page.referrerTabName ? : @"" forKey:@"referrer_tab_name"];
        [dict setObject:page.isPush.intValue ? @(page.isPush.intValue) : @(0) forKey:@"is_push"];
        if (page.inTime.length > 0) {
            // 页面显示时间为空时不记录页面pv事件
            [Phobos track:@"page_view" attributes:dict];
        }
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

- (void)simulativePV:(NSString *)pageName businessId:(NSString *)bid referer:(NSString *)referer{
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    @try {
        // fake（模拟）的事件，所以in与out一样，就是这么规定的
        [dict setObject:[PhobosUtil currentTime] forKey:@"in"];
        [dict setObject:[PhobosUtil currentTime] forKey:@"out"];
        [dict setObject:pageName?:@"" forKey:@"page_name"];
        [dict setObject:bid?:@"" forKey:@"business_id"];
        [dict setObject:referer?:@"" forKey:@"referrer"];
        [dict setObject:@(1) forKey:@"fake"];
        
        [Phobos track:@"page_view" attributes:dict];
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

/**
 *  @brief 将埋点时间封装成词典数据
 *
 *  @since 0.0.1
 */
- (NSDictionary *)prepareDictionaryForEvent:(NSString *)eventName attributes:(NSDictionary *)attributes{
    [self catchNullForEvent:eventName attributes:attributes];
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    @try {
        NSString *currentTime = [PhobosUtil currentTime];
        
        NSMutableDictionary *deviceParams = [NSMutableDictionary new];
        [deviceParams setValue:[[[ASIdentifierManager sharedManager] advertisingIdentifier] UUIDString] forKey:@"idfa"];
        [deviceParams setValue:[[[UIDevice currentDevice] identifierForVendor] UUIDString] forKey:@"idfv"];
        [deviceParams setValue:[PhobosUtil deviceId] forKey:@"device_id"];
        [deviceParams setValue:@"ios" forKey:@"device_type"];
        [deviceParams setValue:@"Apple" forKey:@"manufacturer"];
        [deviceParams setValue:@(self.gps.coordinate.latitude) forKey:@"lat"];
        [deviceParams setValue:@(self.gps.coordinate.longitude) forKey:@"lng"];
        [deviceParams setValue:_netStatus forKey:@"is_WiFi"];
        [deviceParams setValue:[PhobosUtil getIPAddress:YES] forKey:@"ip"];
        [deviceParams setValue:_networkStatus forKey:@"net_type"];
        [deviceParams setValue:[UIDevice platform] forKey:@"model"];
        [deviceParams setValue:[UIDevice currentDevice].systemVersion forKey:@"sys_version"];
        
        NSMutableDictionary *appParams   = [NSMutableDictionary new];
        [appParams setValue:_greyType forKey:@"grey_type"];
        [appParams setValue:_appName forKey:@"name"];
        [appParams setValue:_appVersion forKey:@"version"];
        [appParams setValue:_channelId forKey:@"channel"];
        [appParams setValue:_userType forKey:@"net_type"];
        [appParams setValue:_currentCityId forKey:@"current_city_id"];
        [appParams setValue:@(_serialId++) forKey:@"serial_id"];
        
        if (_signingType == PhobosSigningTypeDebug || _signingType == PhobosSigningTypeRelease) {
            [dict setObject:@(0) forKey:@"is_release"];
        }
        [dict setObject:eventName forKey:@"type"];
        [dict setObject:appParams forKey:@"app"];
        [dict setObject:sdkVersion forKey:@"version"];
        [dict setObject:deviceParams forKey:@"device"];
        [dict setObject:_userId forKey:@"user_id"];
        [dict setObject:currentTime forKey:@"create_at"];
        [dict setObject:attributes forKey:@"params"];
        [dict setObject:_sessionId forKey:@"app_session_id"];
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
    return dict;
}

#pragma mark - helpers
- (void)catchNullForEvent:(NSString *)eventName attributes:(NSDictionary *)attributes {
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        @try {
            for (NSString *key in attributes.allKeys) {
                if ([attributes[key] isMemberOfClass:[NSNull class]]) {
                    if (self.captureNullExpection) {
                        self.captureNullExpection(eventName, attributes);
                    }
                    break;
                }
            }
        } @catch (NSException *exception) {
            
        }
    });
}

#pragma mark - track event handler
+ (void)trackJsEvent:(NSString *)jsonString{
    @try {
        NSData *data = [jsonString dataUsingEncoding:NSUnicodeStringEncoding];
        NSMutableDictionary *dict = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingAllowFragments error:nil];
        
        id pa = dict[@"params"];
        NSDictionary *json;
        if ([pa isKindOfClass:[NSString class]]) {
            NSError *jsonError;
            NSData *objectData = [pa dataUsingEncoding:NSUTF8StringEncoding];
            json = [NSJSONSerialization JSONObjectWithData:objectData
                                                   options:NSJSONReadingMutableContainers
                                                     error:&jsonError];
            [Phobos track:dict[@"type"] attributes:json];
        }else{
            [Phobos track:dict[@"type"]];
        }
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

+ (void)track:(NSString *)eventName{
    [self track:eventName attributes:@{} sendNow:NO currentAPI:_sharedClient.serverAPI];
}

+ (void)track:(NSString *)eventName attributes:(NSDictionary *)attributes{
    [self track:eventName attributes:attributes sendNow:NO currentAPI:_sharedClient.serverAPI];
}

+ (void)track:(NSString *)eventName attributes:(NSDictionary *)attributes sendNow:(BOOL)sendNow{
    [self track:eventName attributes:attributes sendNow:sendNow currentAPI:_sharedClient.serverAPI];
}

+ (void)track:(NSString *)eventName currentAPI:(NSString *)currentAPI {
    [self track:eventName attributes:@{} sendNow:NO currentAPI:currentAPI];
}

+ (void)track:(NSString *)eventName attributes:(NSDictionary *)attributes currentAPI:(NSString *)currentAPI {
    [self track:eventName attributes:attributes sendNow:NO currentAPI:currentAPI];
}

+ (void)track:(NSString *)eventName attributes:(NSDictionary *)attributes sendNow:(BOOL)sendNow currentAPI:(NSString *)currentAPI {
    NSDictionary *dataDict = [_sharedClient prepareDictionaryForEvent:eventName attributes:attributes];
    @try {
        NSData *JSON = [PhobosUtil encodeJSON:dataDict];
        if (sendNow) {
            [_sharedClient sendImmediatelyPhobosWithURL:currentAPI data:dataDict];
        } else{
            [_sharedClient trackPhobosWithURL:currentAPI data:dataDict];
        }
    }
    @catch (NSException *exception) {
        NSAssert(NO, @"哎呀呀,VALUE不能为NSObject ");
    }
}


#pragma mark - 事件存储、发送
/**
 *  普通埋点
 *  @param url 接口url
 *  @param dataArray 埋点数据
 *  @param nowSend 是否需要发送当前缓存数据（app进入到后台、前台等情况）
 */
- (void)sendImmediatelyNormalData {
    [self trackPhobosWithURL:nil data:nil immediate:YES];
}

- (void)trackPhobosWithURL:(NSString *)url data:(NSDictionary *)data {
    [self trackPhobosWithURL:url data:data immediate:NO];
}

- (void)trackPhobosWithURL:(NSString *)url data:(NSDictionary *)data immediate:(BOOL)immediate {
    dispatch_async(_normalQueue, ^{
        dispatch_semaphore_wait(_normalSemaphore, DISPATCH_TIME_FOREVER);
        NSMutableDictionary *dataDict = [[GMCache fetchObjectAtDocumentPathWithkey:PhobosNormalCacheKey] mutableCopy];
        _normalCount++;
        if (!immediate) {
            dataDict = [self dataDict:dataDict setObject:data forKey:url];
            [GMCache storeObjectAtDocumentPathWithkey:PhobosNormalCacheKey object:dataDict];
        }
        if (immediate || self.normalCount >= 50) { // 数据超过一定数量 或 进入后台等逻辑，统一进行发送普通埋点数据
            [dataDict enumerateKeysAndObjectsUsingBlock:^(NSString * _Nonnull key, NSArray * _Nonnull obj, BOOL * _Nonnull stop) {
                if (obj && obj.count > 0) {
                    [self sendImmediatelyPhobosWithURL:key data:obj];
                }
            }];
            [Phobos  removeAllNormalPhobosData];
        }
        dispatch_semaphore_signal(_normalSemaphore);
    });
}

/**
 * 即时发送埋点接口
 * @param url : 接口url
 * @param data : 使用id类型是因为直接调用实时埋点传过来的是Dict,而普通埋点过来的则是Array类型
 */
- (void)sendImmediatelyPhobosWithURL:(NSString *)url data:(id)data {
    dispatch_async(_immediatelyQueue, ^{
        dispatch_semaphore_wait(_immediatelySemaphore, DISPATCH_TIME_FOREVER);
        __block NSMutableDictionary *dataDict = [[GMCache fetchObjectAtDocumentPathWithkey:PhobosImmediatelyCacheKey] mutableCopy];
        dataDict = [self dataDict:dataDict setObject:data forKey:url];
        
        NSInteger allCount = dataDict.allKeys.count;// 请求个数
        __block finishCount = 0;// 请求完成次数,不是请求成功次数，能在最后保证未发送成功的数据，能一次保存下来，等待下次发送
        
        dispatch_semaphore_t semaphore = dispatch_semaphore_create(1);// 因为网络请求是异步回调，需要保证数据安全
        [dataDict enumerateKeysAndObjectsUsingBlock:^(NSString * _Nonnull key, NSArray * _Nonnull obj, BOOL * _Nonnull stop) {
            [self sendDataWithAPI:key data:obj successBlock:^(NSInteger code) {
                dispatch_semaphore_wait(semaphore, DISPATCH_TIME_FOREVER);
                finishCount++;
                if (code == 200) {// 如果数据请求成功，删除当前数据的缓存
                    [dataDict removeObjectForKey:key];
                }
                if (finishCount == allCount) {// 所有数据都请求完成，重新进行数据缓存
                    [GMCache storeObjectAtDocumentPathWithkey:PhobosImmediatelyCacheKey object:dataDict];
                    dispatch_semaphore_signal(_immediatelySemaphore);
                }
                dispatch_semaphore_signal(semaphore);
                phobosLog(@"✈ ---------- ✈ data arrived Mars");
            }];
        }];
    });
}

/**
 * 将数据存入到字典中
 * @param dataDict 需要存入的字典
 * @param object 需要存入的数据
 * @param key 需要将数据存入到dataDict的key所在的数据中
 */
- (NSMutableDictionary *)dataDict:(NSMutableDictionary *)dataDict setObject:(id)object forKey:(NSString *)key {
    if (!dataDict) {
        dataDict = [NSMutableDictionary new];
    }
    // 判断数据类型，统一转换成数组
    NSMutableArray *dataArray = [dataDict.allKeys containsObject:key] ? [dataDict[key] mutableCopy] : [NSMutableArray new];
    NSArray *data = [object isKindOfClass:[NSArray class]] ? object :@[object];
//#ifndef POD_CONFIGURATION_APP_STORE
//    [self verifyPVPhobos:dataArray data:object];
//#endif
    [dataArray addObjectsFromArray:data];
    // 进行数据校验
    @try {
        NSData *json = [PhobosUtil encodeJSON:dataArray];
        [dataDict setValue:dataArray forKey:key];
    } @catch (NSException *exception) {
        NSAssert(NO, @"哎呀呀,VALUE只不能为NSObject ");
    }
    return dataDict;
}

// 校验PV埋点是否有问题，默认返回NO
- (BOOL)verifyPVPhobos:(NSArray *)dataArray data:(id)data {
    if ([data isKindOfClass:[NSDictionary class]]) {
        NSDictionary *dataDict = (NSDictionary *)data;
        if (dataDict[@"type"] && [dataDict[@"type"] isEqualToString:@"page_view"]) {
            NSDictionary *pageParams = dataDict[@"params"];
            long long pageInTime = [pageParams[@"in"] longLongValue];
            long long pageOutTime = [pageParams[@"out"] longLongValue];
            if (pageInTime && pageOutTime) {
                BOOL checkTimeError = NO;
                for (int i = 0; i < dataArray.count; i++) {
                    NSDictionary *obj = dataArray[i];
                    
                    if (obj[@"type"] && [obj[@"type"] isEqualToString:@"page_view"]) {
                        NSDictionary *params = obj[@"params"];
                        long long objInTime = [params[@"in"] longLongValue];
                        long long objOutTime = [params[@"out"] longLongValue];
                        if ( (fabsl(objInTime - pageInTime)) < 1 ||  (fabsl(objOutTime - pageOutTime)) < 1) {
                            phobosLog(@"两个埋点inTime 时间差------%f", (fabsl(objInTime - pageInTime)));
                            phobosLog(@"两个埋点outTime 时间差------%f", (fabsl(objOutTime - pageOutTime)));
                            checkTimeError = YES;
                            phobosLog(@"%s____数据校验失败,PV埋点可能重复", __func__);
                            /*
                            dispatch_async(dispatch_get_main_queue(), ^{
                                NSString *stringTitle = [NSString stringWithFormat:@"%@ PV埋点可能重复", data];
                                UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:stringTitle message:nil delegate:nil cancelButtonTitle:@"确定" otherButtonTitles:nil, nil];
                                [alertView show];
                            });*/
                            break;
                        }
                    }
                }
                return checkTimeError;
            }
        }
    }
    return NO;
}

// 发送埋点数据
- (void)sendDataWithAPI:(NSString *)api data:(NSArray *)dataArray successBlock:(SendDataSuccessBlock)successBlock {
    if (_logEnabled) {
        NSData *data = [NSJSONSerialization dataWithJSONObject:dataArray options:NSJSONWritingPrettyPrinted error:nil];
        NSString *jsonString = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
        phobosLog([NSString stringWithFormat:@"array prepare to fly --✈: %@", jsonString]);
    }
    @try {
        if (self.phobosSendDataBlock) {
            self.phobosSendDataBlock(dataArray);
        }
        NSData *JSON = [PhobosUtil encodeJSON:dataArray];
        NSData *compressedData = [PhobosUtil compressData:JSON];
        if (compressedData) {
            [PhobosUtil sendData:compressedData currentAPI:api success:successBlock];
        }
    }
    @catch (NSException *exception) {
        phobosLog(exception);
    }
}

@end

@implementation Phobos (UtilTest)

/** 获取所有非立即发送埋点数量 */
+ (NSInteger)normalPhobosCount {
    return Phobos.sharedClient.normalCount;
}

/** 获取所有非立即发送埋点数据 */
+ (NSDictionary *)normalPhobosData {
    return [GMCache fetchObjectAtDocumentPathWithkey:PhobosNormalCacheKey];
}

/** 获取url的非立即发送埋点数量 */
+ (NSUInteger)normalPhobosCountForURL:(NSString *)url {
    NSDictionary *dict = [GMCache fetchObjectAtDocumentPathWithkey:PhobosNormalCacheKey];
    NSArray *urlData = dict[url];
    return urlData ? urlData.count : 0;
}

+ (NSArray *)normalPhobosDataForServerAPI {
    return [self normalPhobosDataForURL:Phobos.sharedClient.serverAPI];
}

/** 获取url的非立即发送埋点数据 */
+ (NSArray *)normalPhobosDataForURL:(NSString *)url {
    NSDictionary *dict = [GMCache fetchObjectAtDocumentPathWithkey:PhobosNormalCacheKey];
    return dict[url];
}

+ (NSArray *)immediatelyPhobosForServerAPI {
    return [self immediatelyPhobosDataForURL:Phobos.sharedClient.serverAPI];
}

/** 获取所有立即发送埋点数据 */
+ (NSDictionary *)immediatelyPhobosData {
    return [GMCache fetchObjectAtDocumentPathWithkey:PhobosImmediatelyCacheKey];
}

/** 获取url的立即发送埋点数据 */
+ (NSArray *)immediatelyPhobosDataForURL:(NSString *)url {
    NSDictionary *dict = [GMCache fetchObjectAtDocumentPathWithkey:PhobosImmediatelyCacheKey];
    return dict[url];
}

+ (NSUInteger)immediatelyPhobosCountForURL:(NSString *)url {
    NSDictionary *dict = [GMCache fetchObjectAtDocumentPathWithkey:PhobosImmediatelyCacheKey];
    NSArray *urlData = dict[url];
    return urlData ? urlData.count : 0;
}


/** 清除非立即发送埋点数据缓存 */
+ (void)removeAllNormalPhobosData {
    Phobos.sharedClient.normalCount = 0;
    [GMCache removeObjectAtDocumentPathWithkey:PhobosNormalCacheKey];
}

/** 清除立即发送埋点数据缓存 */
+ (void)removeAllImmediatelyPhobosData {
    [GMCache removeObjectAtDocumentPathWithkey:PhobosImmediatelyCacheKey];
}

- (void)setPhobosSendDataBlock:(void (^)(NSArray *))phobosSendDataBlock {
    objc_setAssociatedObject(self, @selector(phobosSendDataBlock), phobosSendDataBlock, OBJC_ASSOCIATION_COPY_NONATOMIC);
}

- (void (^)(NSArray *))phobosSendDataBlock {
    return objc_getAssociatedObject(self, @selector(phobosSendDataBlock));
}

@end
